$Id: ADT.lhs,v 3.3 2003/04/03 14:26:16 afc Exp $
$Log: ADT.lhs,v $
Revision 3.3  2003/04/03 14:26:16  afc
Added IntervalList/Notexisting, unusedPred/newvarPred and
NewVar/Interval

Revision 3.2  2002/07/16 15:50:42  ajc99
moved AncestorFM types into here.

Revision 3.1  2002/05/17 15:31:23  ajc99
got rid of UserTypes.hs. made supp/conf settings, not hardcoded.
removed old Class stuff.

Revision 3.0  2002/04/04 10:39:24  ajc99
Now uses hierarchical background knowledge.

Revision 2.2  2002/02/06 14:27:13  ajc99
Uses PackedString. Tidied code.

Revision 2.1  2001/12/07 16:17:27  ajc99
Removed classes from QueryTree. Now does real associations.

Revision 2.0  2001/12/04 18:37:14  ajc99
more deepseq instances, minor changes.

Revision 1.7  2001/11/13 15:34:35  ajc99
Examples now have predicates grouped with indices.

Revision 1.6  2001/11/12 18:04:33  ajc99
Added instances for DeepSeq

Revision 1.5  2001/11/06 17:58:01  ajc99
added ConstraintNum.

Revision 1.4  2001/11/06 15:48:28  ajc99
allow constants generated from bg file in modes.

Revision 1.3  2001/10/18 17:01:18  ajc99
Added QueryTree type.

Revision 1.2  2001/10/12 13:41:43  ajc99
moved Binding type here

Revision 1.1  2001/09/27 15:04:49  ajc99
Initial revision



> module ADT(module ADT) where

> import FiniteMap
> import PackedString(PackedString)
> import Ratio

> import DeepSeq

> defaultSupport,defaultConfidence :: Ratio Integer
> defaultSupport    = 0.1  
> defaultConfidence = 0.3 

Should be able to remove the "deriving Show"s later

> data Mode    = Mode String [ModeArg] deriving (Show,Read) 
> data ModeNum = ModeNum Int String [ModeArg] deriving (Show,Read) 
> data ModeArg = Plus | Minus | PlusMinus | NotExisting 
>                | ConstList [String] | ConstPred String | ConstTree String
>		 | IntervalList [(Double,Double)]
>   deriving (Show,Read)


> data Type    = Type String [String] deriving (Read,Show) 
> data TypeNum = TypeNum Int String [String] deriving (Read,Show) 


> data PredString  = PredString String [Arg] deriving Show 
>                              -- functor, args
> data Pred        = Pred !Int [Arg] deriving (Show,Eq) -- functor, args
> data Preds       = Preds MaxVar [Pred] deriving Show -- maxvar, preds
> data PredStrings = PredStrings MaxVar [PredString] deriving Show
>                    -- maxvar, preds

> unusedPred :: Int
> unusedPred = -1
> newVarPred :: Int
> newVarPred = -2


> data Arg         = Atom !PackedString | Var !Int | NewVar !Int
>                    | ArgTreeName !String | ArgTree !(String,[HierTree])
>		     | Interval !Double !Double 
>        deriving (Show, Eq, Ord)

> type RuleHead    = Maybe Int


> type IndexedPredList = [(Int,[Pred])]
> type ExampleString   = [PredString]
> type Example         = IndexedPredList
> type BGString        = [PredString]
> type PlainBG         = IndexedPredList
> type BG	       = (PlainBG,HierTrees)

> data Rule = Rule Integer Integer Integer Integer Pred [Pred] deriving Show 
>             -- supp, conf
> data Assoc      = Assoc [Pred] Integer Integer deriving Show -- support 


> type Binding  = [(Arg,Arg)]
> type Bindings = [Binding]

> data Constraint     = ConstraintPair (String, String) 
>                       | ConstraintLim String Int
> type Constraints    = [Constraint]
> data ConstraintNum  = ConstraintNumPair (Int, Int)
>                       | ConstraintNumLim Int Int
> type ConstraintNums = [ConstraintNum]

> data Setting =   M [Mode]            -- modes
>                | T [Type]            -- types
>                | Co Constraints      -- constraints
>                | K Mode              -- key
>		 | RH String	       -- rule head
>		 | Sp (Ratio Integer)  -- support
>		 | Cf (Ratio Integer)  -- confidence


> data QueryTree =   QTNode Pred !Integer !Integer !MaxVar [QueryTree] 
>                  | QTNil  deriving (Show,Eq) -- supp, no of examples


> data HierTree = HierNode !Arg !Integer !Bool [HierTree]   -- actually only Atom
>                 | HierNil
>      deriving (Show,Eq,Ord)

> newtype HierTreePair = HierTreePair (String,[HierTree]) deriving Show
> type HierTrees = [HierTreePair]


> type MaxVar = Int 

> type AncestorFMs = [AncestorFM]
> type AncestorFM = FiniteMap (Arg,Arg) Bool



----------------------------------------------------------
accessor functions

> getArgTypes types num =
>   if num == newVarPred then [] else getArgTypes2 types num
> getArgTypes2 [] num = error ("ADT.lhs: getArgTypes: No type for <" 
>                      ++ show num ++ ">\n")
> getArgTypes2 ((TypeNum tnum tname args):rest) num 
>      = if tnum == num then args
>        else getArgTypes rest num

> getPArgs (Pred pname args) = args
> getPName (Pred pname args) = pname


> getModes :: [Setting] -> [Mode]
> getModes [] = error "No modes defined"
> getModes ((M m) : _) = m
> getModes (_:rest) = getModes rest

> getTypes [] = error "No types defined"
> getTypes ((T t): _) = t
> getTypes (_:rest) = getTypes rest

> getKey [] = error "No key defined"
> getKey ((K k): _) = k
> getKey (_:rest) = getKey rest

> getConstraints [] = error "No constraints defined"
> getConstraints ((Co c): _) = c
> getConstraints (_:rest) = getConstraints rest

> getRuleHead [] = Nothing
> getRuleHead ((RH rh): _) = Just rh
> getRuleHead (_:rest)     = getRuleHead rest

> getSupport [] = defaultSupport
> getSupport ((Sp sp): _) = sp
> getSupport (_:rest)     = getSupport rest

> getConfidence [] = defaultConfidence
> getConfidence ((Cf cf): _) = cf
> getConfidence (_:rest)     = getConfidence rest


> notQTNil QTNil  = False
> notQTNil _      = True 

> isQTNil QTNil   = True
> isQTNil _       = False

> isAtom (Atom _) = True
> isAtom _        = False

> lookupHierTree :: String -> HierTrees -> Maybe [HierTree]
> lookupHierTree name [] = Nothing
> lookupHierTree name ((HierTreePair (s,ts)):rest) = 
>    if name == s then Just ts else lookupHierTree name rest 

> fstHierTreePair (HierTreePair (x,y)) = x

> isArgTree (ArgTree t) = True
> isArgTree _           = False

> isConstTree (ConstTree t) = True
> isConstTree _             = False

> notHierNil HierNil = False
> notHierNil _       = True

-------------------------------------------------------------------------


instances of DeepSeq (see DeepSeq.lhs). This module provides strict 
sequencing of "deep" structures, to stop Haskell being lazy.

> instance DeepSeq Mode where
>   deepSeq (Mode name args) y = deepSeq name $ deepSeq args y
> instance DeepSeq ModeNum where
>   deepSeq (ModeNum num name args) y = 
>        deepSeq num $ deepSeq name $ deepSeq args y
> instance DeepSeq ModeArg where
>   deepSeq Plus y               = y
>   deepSeq Minus y              = y
>   deepSeq PlusMinus y          = y
>   deepSeq (ConstList consts) y = deepSeq consts y
>   deepSeq (ConstPred cp) y     = deepSeq cp y
>   deepSeq (ConstTree ct) y     = deepSeq ct y
>   deepSeq (IntervalList consts) y = deepSeq consts y

> instance DeepSeq Pred where
>   deepSeq (Pred n args) y = deepSeq n $ deepSeq args y
> instance DeepSeq Arg where
>   deepSeq (Atom s) y         = deepSeq s y
>   deepSeq (Var i)  y         = deepSeq i y
>   deepSeq (ArgTreeName n) y  = deepSeq n y
>   deepSeq (ArgTree (n,ts)) y = deepSeq n $ deepSeq ts y
>   deepSeq (Interval b e) y   = deepSeq b $ deepSeq e y

> instance DeepSeq QueryTree where
>   deepSeq QTNil y = y
>   deepSeq (QTNode pred supp nexs maxvar qtrees) y = 
>        deepSeq pred $ deepSeq supp $ deepSeq nexs $ deepSeq maxvar 
>           $ deepSeq qtrees y

> instance DeepSeq PackedString where

> instance DeepSeq HierTree where
>   deepSeq HierNil y = y
>   deepSeq (HierNode name v b ts) y = 
>        deepSeq name $ deepSeq v $ deepSeq b $ deepSeq ts y
