$Id: BGKBParser.ly,v 3.2 2002/07/16 15:51:35 ajc99 Exp $
$Log: BGKBParser.ly,v $
Revision 3.2  2002/07/16 15:51:35  ajc99
bugfix of case where Preds are empty

Revision 3.1  2002/05/17 15:32:45  ajc99
allows names to contain hyphens, dots and dashes.

Revision 3.0  2002/04/04 10:40:52  ajc99
Now uses hierarchical background knowledge.

Revision 2.1  2002/02/06 15:13:18  ajc99
added PackedString

Revision 2.0  2001/12/04 18:36:03  ajc99
no change, just version change.

Revision 1.2  2001/11/05 17:13:31  ajc99
added comment parsing

Revision 1.1  2001/09/27 15:05:34  ajc99
Initial revision



> {
> module BGKBParser where
> import Char
> import ADT
> import PackedString(packString)
> }

> %name kbparser Exs
> %name bgparser BG
> %tokentype { Token }

> %monad { P } { thenP } { returnP }
> %lexer { lexer } { TokenEOF }

> %token 
>       name		{ TokenName $$ }
>	','		{ TokenComma }
>	'('		{ TokenOB }
>	')'		{ TokenCB }
>	'['		{ TokenOS }
>	']'		{ TokenCS }
>       '.'             { TokenDot }
>	'@'		{ TokenAt }
>	hierarchy	{ TokenHierarchy }

> %%

> Exs	  : Preds			{ if null $1 then [] else [$1] }
>	  | Preds '@' Exs		{ $1 : $3 }

> Preds   : {- empty -}			{ [] }
>         | Preds1			{ $1 }

> Preds1  : Pred			{ [$1] }
>         | Pred Preds1			{ $1 : $2 } 

> Pred    : name '(' ArgList ')' '.'    { PredString $1 $3 }

> ArgList : name                        { [Atom (packString $1)] }
>         | name ',' ArgList            { (Atom (packString $1)) : $3 }


> BG	  : Preds Hiers			{ ($1,$2) }

> Hiers   : {- empty -}			{ [] }
>         | Hiers1			{ $1 }

> Hiers1  : Hier			{ [$1] }
>	  | Hier Hiers1			{ $1 : $2 }

> Hier	  : hierarchy '(' name ',' '[' TreeList ']' ')' '.'    { HierTreePair ($3, $6) }

> Tree	  : name		       { HierNode (Atom (packString $1)) 0 False [] }
>	  | name '[' TreeList ']'      { HierNode (Atom (packString $1)) 0 False $3 }

> TreeList : Tree			{ [$1] }
>	   | Tree ',' TreeList		{ $1 : $3 }

> {

-----------------------------------------------------------------------------
The monad serves three purposes: 

	* it passes the input string around
	* it passes the current line number around
	* it deals with success/failure.

> data ParseResult a
>	= ParseOk a
>	| ParseFail String

> -- text, line number, filename 
> type P a = String -> Int -> String -> ParseResult a

> thenP :: P a -> (a -> P b) -> P b
> m `thenP` k = \s l n -> 
>	case m s l n of
>		ParseFail s -> ParseFail s
>		ParseOk a -> k a s l n

> returnP :: a -> P a
> returnP a = \s l n -> ParseOk a

-----------------------------------------------------------------------------

The datastructure for the tokens...

> data Token
>	= TokenName String
>	| TokenOB
>	| TokenCB
>	| TokenOS
>	| TokenCS
>	| TokenComma
>	| TokenDot
>	| TokenEOF
>	| TokenAt
>	| TokenHierarchy

.. and a simple lexer that returns this datastructure.

> lexer :: (Token -> P a) -> P a
> lexer cont s = case s of
> 	[] -> cont TokenEOF []
>  	('\n':cs) -> \line -> lexer cont cs (line+1)
>	('%':cs) -> lexer cont (dropWhile ((/=) '\n') cs)	
> 	('.':cs) -> cont TokenDot cs
> 	(',':cs) -> cont TokenComma cs
> 	('(':cs) -> cont TokenOB cs
> 	(')':cs) -> cont TokenCB cs
> 	('[':cs) -> cont TokenOS cs
> 	(']':cs) -> cont TokenCS cs
>	('@':cs) -> cont TokenAt cs
> 	(c:cs) 
>               | isSpace c -> lexer cont cs
>               | isFirstNameChar c -> lexVar (c:cs)
> 		| otherwise -> \line name -> error (name ++ ": character '" ++ [c] ++ "' not recognised at line " ++ show line ++ "\n")
>  where
> 	lexVar cs =
>    	    case span isNameChar cs of
>		("hierarchy",rest) -> cont TokenHierarchy rest
> 		(var,rest)   -> cont (TokenName var) rest

> isFirstNameChar c = isDigit c || isAlpha c || c == '_'
> isNameChar c = isFirstNameChar c || c == '-' || c == '.' || c == '/'

> runKBParser :: String -> String -> [ExampleString]
> runKBParser s filename = 
>     case kbparser s 1 filename of
>		       ParseOk e -> e
>		       ParseFail s -> error (filename ++ ": " ++ s)

> runBGParser :: String -> String -> (BGString,HierTrees)
> runBGParser s filename = 
>     case bgparser s 1 filename of
>		       ParseOk e -> e
>		       ParseFail s -> error (filename ++ ": " ++ s)

-----------------------------------------------------------------------------
The following functions should be defined for all parsers.

This is the overall type of the parser.

> type ParseEx = P [ExampleString]
> kbparser :: ParseEx

> type ParseBG = P (BGString,HierTrees)
> bgparser :: ParseBG

The next function is called when a parse error is detected.  It has
the same type as the top-level parse function.

> happyError :: P a
> happyError = \s i n -> error (
>	n ++ ": Parse error in line " ++ show (i::Int) ++ "\n")

-----------------------------------------------------------------------------


> }