$Id: Dumper.lhs,v 3.3 2003/04/03 14:10:19 afc Exp $
$Log: Dumper.lhs,v $
Revision 3.3  2003/04/03 14:10:19  afc
mergeTree now adds all counts, not just leaves.

Revision 3.2  2002/07/16 16:22:31  ajc99
Added facilities for reading/writing infreqs.

Revision 3.1  2002/05/17 15:37:11  ajc99
removed redundant Classes stuff.

Revision 3.0  2002/04/04 10:42:39  ajc99
Now uses hierarchical background knowledge.

Revision 2.3  2002/02/26 11:18:11  ajc99
Added existsQueryFile

Revision 2.2  2002/02/06 15:15:42  ajc99
Now uses Julian's Sequential code.

Revision 2.1  2001/12/07 16:18:51  ajc99
Removed classes from querytree. Now merges trees with proper counts
all down trunk.

Revision 2.0  2001/12/04 18:20:06  ajc99
First version of Dumper, for dumping data to files for exchange
between beowulf nodes.


> module Dumper( dumpData, undumpData, dumpQueries, undumpQueries,
>                mergeTree, existsQueryFile, readInfreqs, writeInfreqs )
> where

> import IO
> import Directory (doesFileExist)

> import ADT
> import DeepSeq
> import FromJulian


> dumpData :: String -> String -> String -> [ModeNum] -> BG -> Mode 
>              -> QueryTree -> IO()              
> dumpData filename workingdir level modenums bg key queries = 
>   let short_filename = workingdir ++ "/" ++ (removepath filename) 
>                        ++ "." ++ level ++ ".data" in  
>   do
>    dumpMainData short_filename modenums bg key 
>    dumpQueries (filename ++ "." ++ level ++ ".data") workingdir queries


> dumpMainData filename modenums bg key =
>    do bh <- openFile filename WriteMode 
>       toSeq bh modenums
>       toSeq bh bg
>       toSeq bh key
>       hClose bh


> undumpData :: String -> String -> String 
>               -> IO(([ModeNum],BG,Mode,QueryTree))
> undumpData filename workingdir level = 
>   let short_filename = workingdir ++ "/" ++ (removepath filename) 
>                        ++ "." ++ level ++ ".data" in  
>    do 
>       (mns,bg,key) <- undumpMainData short_filename
>       queries <- undumpQueries (filename ++ "." ++ level ++ ".data") 
>                  workingdir
>       return (mns,bg,key,queries)



> undumpMainData filename =
>      do fh  <- openFile filename ReadMode
>         mns <- fromSeq fh
>         bg  <- fromSeq fh
>         key <- fromSeq fh
>         hClose fh
>         return (mns,bg,key)

-----------------------------------------------------------------------

> readInfreqs :: String -> String -> Int -> IO( [[Pred]] )
> readInfreqs filestem workingdir level = 
>  if level==1 then return []
>  else do h <- openFile (workingdir ++ "/" ++ (removepath filestem) ++ ".infreqs." ++ (show (level - 1))) ReadMode
>          infreqs <- fromSeq h
>          hClose h
>          return infreqs

> writeInfreqs :: String -> String -> Int -> [[Pred]] -> IO()
> writeInfreqs filestem workingdir level infreqs =
>   do h <- openFile (workingdir ++ "/" ++ (removepath filestem) ++ ".infreqs." ++ show level) WriteMode
>      toSeq h infreqs 
>      hClose h


-----------------------------------------------------------------------

> undumpQueries filename workingdir =
>    let qfilename = workingdir ++ "/" ++ "Q" ++ (removepath filename) in
>    do ok <- doesFileExist qfilename
>       if ok 
>         then string2queries qfilename
>         else error ("Missing file " ++ qfilename ++ "\n")

used to be
 --   readFile qfilename >>= \text -> return $!! (string2queries text)

> dumpQueries filename workingdir queries =
>    let qfilename = workingdir ++ "/" ++ "Q" ++ (removepath filename) in
>    queries2string qfilename queries

 queries2string :: QueryTree -> String 
 queries2string qt = show qt 

> queries2string :: String -> QueryTree -> IO() 
> queries2string filename queries = 
>    do fh <- openFile filename WriteMode
>       toSeq fh queries
>       hClose fh

 string2queries :: String -> IO(QueryTree)
 string2queries s = 
       do
            text <- readFile s
            return (runQueryParser text s)

> string2queries :: String -> IO(QueryTree)
> string2queries filename = 
>      do fh   <- openFile filename ReadMode
>	  tree <- fromSeq fh
>         hClose fh
>         return tree


> removepath :: String -> String
> removepath = reverse . takeWhile ((/=) '/') . reverse 

------------------------------------------------------------------------

> existsQueryFile :: String -> String -> IO( Bool )
> existsQueryFile filename workingdir =
>    let qfilename = workingdir ++ "/" ++ "Q" ++ (removepath filename) in
>    doesFileExist qfilename

------------------------------------------------------------------------

> mergeTree :: QueryTree -> QueryTree -> QueryTree

> mergeTree QTNil QTNil = QTNil

 > mergeTree q1@(QTNode p1 s1 n1 maxvar1 []) 
 >                               q2@(QTNode p2 s2 n2 maxvar2 []) =
 >    if getPName p1 == getPName p2 && maxvar1 == maxvar2 then
 >       QTNode (mergePreds p1 p2) (s1+s2) (n1+n2) maxvar1 []
 >    else error ("Can't merge different trees:\n " ++ (show q1) 
 >                         ++ "\n\n" ++ (show q2) ++ "\n")

> mergeTree q1@(QTNode p1 s1 n1 maxvar1 qts1) 
>                               q2@(QTNode p2 s2 n2 maxvar2 qts2) =
>    if getPName p1 == getPName p2 && maxvar1 == maxvar2 then
>       QTNode (mergePreds p1 p2) (s1+s2) (n1+n2) maxvar1 
>              (zipWith mergeTree qts1 qts2)
>    else error ("Can't merge different trees:\n " ++ (show q1) 
>                         ++ "\n\n" ++ (show q2) ++ "\n")
> mergeTree q1 q2 = error ("Can't merge different trees:\n " ++ (show q1) 
>                         ++ "\n\n" ++ (show q2) ++ "\n")


-----------------------------------------------------------------------

> mergePreds (Pred name1 args1) (Pred name2 args2)
>    = Pred name1 (zipWith mergeArgs args1 args2)

> mergeArgs (ArgTree (name1,ts1)) (ArgTree (name2,ts2)) =
>      ArgTree (name1,zipWith mergeHierTrees ts1 ts2)
> mergeArgs x y = x

> mergeHierTrees (HierNode name1 val1 b1 ts1) (HierNode name2 val2 b2 ts2) =
>   HierNode name1 (val1+val2) False (zipWith mergeHierTrees ts1 ts2)

