$Id: FileIO.lhs,v 3.1 2002/05/17 15:40:36 ajc99 Exp $
$Log: FileIO.lhs,v $
Revision 3.1  2002/05/17 15:40:36  ajc99
supp/conf now settings.

Revision 3.0  2002/04/04 10:43:19  ajc99
Now uses hierarchical background knowledge.

Revision 2.2  2002/02/06 15:18:11  ajc99
Added PackedString

Revision 2.1  2001/12/07 16:24:47  ajc99
allows more advanced constraints, and a rulehead setting.

Revision 2.0  2001/12/04 18:33:13  ajc99
no change, just version number.

Revision 1.3  2001/11/13 15:34:03  ajc99
Examples now have predicates grouped with indices.

Revision 1.2  2001/11/12 18:01:17  ajc99
moved compilekb,compilebg, etc into here.

Revision 1.1  2001/09/27 15:07:29  ajc99
Initial revision



> module FileIO(readKnowledgeBase, readBackground, readSettings,
>	        compileKB,compileBG,compileModes,compileTypes,
>		compileConstraints,compileRuleHead,expandModeConsts)

> where

> import List(nub)
> import PackedString(unpackPS)
> import Ratio(Ratio)

> import ADT
> import BGKBParser
> import SettingsParser

> readKnowledgeBase filename = 
>      readKB (filename ++ ".kb")

> readBackground filename =
>      readBG (filename ++ ".bg")

> readSettings :: String 
>              -> IO ([Mode],[Type],Mode,Constraints,Maybe String,
>                     Ratio Integer, Ratio Integer)
> readSettings filestem =
>      let filename = filestem ++ ".s" in
>      do 
>         text <- readFile filename
>	  let settings = runSettingsParser text filename
>             modes     = getModes       settings
> 	      types     = getTypes       settings
>	      key       = getKey         settings
>	      constrs   = getConstraints settings
>	      rulehead  = getRuleHead    settings
>	      min_supp  = getSupport	 settings
>	      min_conf  = getConfidence  settings
>         return (modes,types,key,constrs,rulehead,min_supp,min_conf)



background knowledge

> readBG :: String -> IO(BGString,HierTrees)
> readBG bgfile =
>	 do
>           text <- readFile bgfile
>           return (runBGParser text bgfile)


examples

> readKB :: String -> IO([ExampleString])
> readKB kbfile =
>	 do
>           text <- readFile kbfile
>           return (runKBParser text kbfile)

 

-------------------------------------------------------------

Compiling the KB

> compileKB :: [ModeNum] -> Mode -> ExampleString -> Example
> compileKB modes key example = 
>    foldl groupPreds [(0,[])] $ 
>    (map (prednamestonums modes key) example)

brings the key data to the top of each example, so we don't have to
search through the whole example later to find it.


> groupPreds [] pred@(Pred pnum pargs) = [(pnum,[pred])]
> groupPreds (g@(gnum,ps):rest) pred@(Pred pnum pargs) = 
>      if gnum == pnum then (gnum,pred:ps):rest
>      else g:groupPreds rest pred


-------------------------------------------------------------------

Compiling the BG

> compileBG :: [ModeNum] -> Mode -> (BGString,HierTrees) -> BG
> compileBG modes key (bg,hiers) = 
>    (foldl groupPreds [] $
>     map (prednamestonums modes key) bg, hiers)
> --     map (treenamestonums modes key) hiers)

> prednamestonums :: [ModeNum] -> Mode -> PredString -> Pred
> prednamestonums modenums (Mode kname kargs) (PredString name args) = 
>    if name == kname then Pred 0 args
>    else case lookupModeNum name modenums of 
>              Just n -> Pred n args
>              Nothing -> Pred (-1) args

> treenamestonums :: [ModeNum] -> Mode -> (String,HierTrees) -> (Int,HierTrees)
> treenamestonums modenums (Mode kname kargs) (name,t) = 
>    if name == kname then (0,t)
>    else case lookupModeNum name modenums of 
>              Just n -> (n,t)
>              Nothing -> (-1,t)

-------------------------------------------------------------------

Compiling the modes

> compileModes :: Mode -> [Mode] -> [ModeNum]
> compileModes key modes =
>   let name2num = zip (nub (map getmodename (key:modes))) [0..] 
>       getmodename (Mode name args) = name
>       makeModeNum name2num (Mode name args) = 
>           let Just n = lookup name name2num in ModeNum n name args
>   in 
>   map (makeModeNum name2num) modes

Expanding constants from background knowledge into modes
Assumes constant generating predicate will have arity 1

> expandModeConsts :: BGString -> ModeNum -> ModeNum
> expandModeConsts bg (ModeNum n name args) =
>   ModeNum n name (map (expandConsts bg) args)

> expandConsts bg (ConstPred name) = 
>     let unAtom (Atom a) = unpackPS a
>         isPred name (PredString pname args) = pname == name
>     in
>      ConstList (concatMap ( \(PredString pname args) -> (map unAtom args)) 
>                 (filter (isPred name) bg))
> expandConsts bg x = x



-------------------------------------------------------------------

Compiling the types

> compileTypes :: Mode -> [ModeNum] -> [Type] -> [TypeNum]
> compileTypes key modes [] = [] 
> compileTypes key@(Mode kname kargs) modes ((Type name args):rest) = 
>    typenum : compileTypes key modes rest
>    where 
>       typenum = if name == kname then TypeNum 0 name args
>                 else 
>                     case lookupModeNum name modes of 
>                        Just num -> TypeNum num name args 
>                        Nothing  -> TypeNum (-1) name args 

> lookupModeNum name [] = Nothing
> lookupModeNum name ((ModeNum mnum mname margs):modenums) =
>   if name == mname then Just mnum else lookupModeNum name modenums

> compileRuleHead modes Nothing = Nothing
> compileRuleHead modes (Just name) = 
>     case lookupModeNum name modes of 
>        Just num -> Just num
>        Nothing  -> error ("compileRuleHead: can't find " ++ name 
>                           ++ " in " ++ (show modes) ++ "\n")

----------------------------------------------------------------------

Compiling the constraints

> compileConstraints :: [ModeNum] -> Constraint -> ConstraintNum
> compileConstraints modes (ConstraintPair (x,y)) = 
>    let xn = case lookupModeNum x modes of 
>               Just xnum -> xnum
>		Nothing   -> -1
>	 yn = case lookupModeNum y modes of 
>	        Just ynum -> ynum
>		Nothing   -> -1
>   in ConstraintNumPair (xn,yn)
> compileConstraints modes (ConstraintLim x y) = 
>    let xn = case lookupModeNum x modes of 
>               Just xnum -> xnum
>		Nothing   -> -1
>   in ConstraintNumLim xn y

