{-
$Id: FromJulian.hs,v 3.2 2003/04/03 13:43:09 afc Exp $
$Log: FromJulian.hs,v $
Revision 3.2  2003/04/03 13:43:09  afc
Added Interval, NotExisting and Double.

Revision 3.1  2002/05/17 15:41:23  ajc99
ClassTree stuff removed.

Revision 3.0  2002/04/04 10:43:52  ajc99
Now uses hierarchical background knowledge.

Revision 1.1  2002/02/06 15:18:55  ajc99
Initial revision


-}

module FromJulian where

import IO
import Char	( chr, ord )
import ADT
import PackedString(PackedString,unpackPS,packString)

{-
testBG = Node "hello" [Node "bye" [CNil]]

main = do bh <- openFile "binfile" WriteMode
          toSeq bh testBG
          hClose bh
          bh2 <- openFile "binfile" ReadMode
	  m <- fromSeq bh2
	  putStrLn (show (m::ClassTree))
-}

panic msg = error ("panic: " ++ msg)

putC :: Handle -> Char -> IO ()
putC = hPutChar
putS :: Handle -> String -> IO ()
putS = hPutStr
getC :: Handle -> IO Char
getC = hGetChar
getS :: Handle -> Int -> IO String
getS h n
   | n < 0     = panic "getS: negative"
   | n == 0    = return []
   | otherwise = do c <- getC h
                    cs <- getS h (n-1)
                    return (c:cs)

-- write the length of an item using an expanding code.  This makes
-- it possible to implement a reader with zero lookahead.  0-9 are
-- emitted as-is.  Otherwise emit 'A' + length (show n) ++ show n
emitLen :: Handle -> Int -> IO ()
emitLen h n
   | n < 0 = panic "emitLen"
   | n < 10 = putC h (chr (n + ord '0'))
   | otherwise
   = let ns = show n
         nd = length ns
     in  putC h (chr (nd + ord 'A')) >> putS h ns

-- read a len.  Take care to force the returned Int before returning it.
readLen :: Handle -> IO Int
readLen h
   = do c1 <- getC h
        if c1 >= '0' && c1 <= '9'
         then do let n = ord c1 - ord '0'
                 n `seq` return n
         else do let nd = ord c1 - ord 'A'
                 str <- getS h nd
                 let n = read str
                 n `seq` return n


class Sequential t where
   toSeq   :: Handle -> t -> IO ()
   fromSeq :: Handle -> IO t

-- Alas, this encompasses String == [Char] in a way we don't want
-- so use toSeqString and fromSeqString for String

-- instance Sequential [String] where
toSeqSS :: Handle -> [String] -> IO()
fromSeqSS :: Handle -> IO [String]
toSeqSS h xs
      = let n = length xs
        in  -- just wop 'em in back-to-back, since the reader knows how
            -- many to expect
            emitLen h n >> mapM_ (toSeqString h) xs
fromSeqSS h
      = do nxs <- readLen h
           loop nxs
        where
           loop :: Int -> IO [String]
           loop 0 = return []
           loop n = do x <- fromSeqString h
                       xs <- loop (n-1)
                       return (x:xs)

-- instance Sequential [PackedString] where
toSeqPSSS :: Handle -> [PackedString] -> IO()
fromSeqPSSS :: Handle -> IO [PackedString]
toSeqPSSS h xs
      = let n = length xs
        in  -- just wop 'em in back-to-back, since the reader knows how
            -- many to expect
            emitLen h n >> mapM_ (toSeqPSString h) xs
fromSeqPSSS h
      = do nxs <- readLen h
           loop nxs
        where
           loop :: Int -> IO [PackedString]
           loop 0 = return []
           loop n = do x <- fromSeqPSString h
                       xs <- loop (n-1)
                       return (x:xs)

instance Sequential a => Sequential [a] where
   toSeq h xs
      = let n = length xs
        in  -- just wop 'em in back-to-back, since the reader knows how
            -- many to expect
            emitLen h n >> mapM_ (toSeq h) xs
   fromSeq h
      = do nxs <- readLen h
           loop nxs
        where
           loop :: Sequential b => Int -> IO [b]
           loop 0 = return []
           loop n = do x <- fromSeq h
                       xs <- loop (n-1)
                       return (x:xs)

instance Sequential Int where
   toSeq h i = let str = show i in emitLen h (length str) >> putS h str
   fromSeq h = do nd <- readLen h
                  str <- getS h nd
                  let n = read str
                  n `seq` return n

instance Sequential Integer where
   toSeq h i = let str = show i in emitLen h (length str) >> putS h str
   fromSeq h = do nd <- readLen h
                  str <- getS h nd
                  let n = read str
                  n `seq` return n

instance Sequential Double where
   toSeq h d = let str = show d in emitLen h (length str) >> putS h str
   fromSeq h = do nd <- readLen h
                  str <- getS h nd
                  let n = read str
                  n `seq` return n

instance (Sequential a, Sequential b) => Sequential (a,b) where
   toSeq h (x,y) = toSeq h x >> toSeq h y
   fromSeq h = do x <- fromSeq h
                  y <- fromSeq h
                  return (x,y)

--instance Sequential String where   {- see comment above -}
toSeqString   h str = emitLen h (length str) >> putS h str
fromSeqString h     = do nd <- readLen h
                         str <- getS h nd
                         return str

--instance Sequential PackedString where   {- see comment above -}
toSeqPSString :: Handle -> PackedString -> IO()
fromSeqPSString :: Handle -> IO(PackedString)
toSeqPSString   h str = let ustr = unpackPS str in
                        emitLen h (length ustr) >> putS h ustr
fromSeqPSString h     = do nd <- readLen h
                           str <- getS h nd
			   let pstr = packString str 
                           pstr `seq` return pstr


test = do h <- openFile "testfile" ReadMode
          xs <- fromSeq h
          hClose h
          putStrLn (show (xs :: [Int]))

--------------------------------------------------------------

instance Sequential ModeArg where
  toSeq h (Plus) = 
   putC h 'P'
  toSeq h (Minus) =
   putC h 'M'
  toSeq h (PlusMinus) =
   putC h 'B'  
  toSeq h (NotExisting) =
   putC h 'N'  
  toSeq h (ConstList ss) =
   do putC h 'L'
      toSeqSS h ss
  toSeq h (ConstPred s) =
   do putC h 'R'
      toSeqString h s
  toSeq h (ConstTree s) =
   do putC h 'T'
      toSeqString h s
  toSeq h (IntervalList ss) =
   do putC h 'I'
      toSeq h ss
  fromSeq h =
    do i <- getC h 
       case i of
         'P' -> return Plus
         'M' -> return Minus
         'B' -> return PlusMinus
         'N' -> return NotExisting
         'L' -> do ss <- fromSeqSS h; return (ConstList ss)
         'R' -> do s  <- fromSeqString h; return (ConstPred s)
         'T' -> do s  <- fromSeqString h; return (ConstTree s)
         'I' -> do ss <- fromSeq h; return (IntervalList ss)



instance Sequential ModeNum where
 toSeq h (ModeNum i s args) =
   do
      toSeq h i
      toSeqString h s
      toSeq h args
 fromSeq h =
   do i    <- fromSeq h 
      s    <- fromSeqString h
      args <- fromSeq h 
      let m = ModeNum i s args
      m `seq` return m

instance Sequential Mode where
 toSeq h (Mode s args) =
   do 
      toSeqString h s
      toSeq h args
 fromSeq h =
   do s    <- fromSeqString h 
      args <- fromSeq h
      let m = Mode s args 
      m `seq` return m



instance Sequential Arg where
 toSeq h (Atom a) =
   do putC h 'A'
      toSeqPSString h a
 toSeq h (Interval b e) =
   do putC h 'I'
      toSeq h b
      toSeq h e
 toSeq h (Var i) = 
   do putC h 'V'
      toSeq h i
 toSeq h (NewVar i) = error ("trying to write a NewVar with toSeq\n")
 toSeq h (ArgTreeName n) =
   do putC h 'N'
      toSeqString h n
 toSeq h (ArgTree (name,ts)) =
   do putC h 'T'
      toSeqString h name
      toSeq h ts
 fromSeq h =
   do c <- getC h  
      case c of 
         'A' -> do a <- fromSeqPSString h
		   let g = Atom a
		   g `seq` return g
         'I' -> do b <- fromSeq h
		   e <- fromSeq h	
		   let g = Interval b e 
		   g `seq` return g
         'V' -> do i <- fromSeq h
		   let g = Var i
		   g `seq` return g
         'N' -> do n <- fromSeqString h
		   let g = ArgTreeName n
		   g `seq` return g
         'T' -> do name <- fromSeqString h
                   ts   <- fromSeq h
		   let g = ArgTree (name,ts)
		   g `seq` return g


instance Sequential Pred where
 toSeq h (Pred i args) = 
   do toSeq h i
      toSeq h args
 fromSeq h =
   do i    <- fromSeq h 
      args <- fromSeq h 
      let p = Pred i args 
      p `seq` return p

instance Sequential QueryTree where
 toSeq h (QTNil) = 
   putC h 'L'
 toSeq h (QTNode pred supp n mv ts) =
   do putC h 'N'
      toSeq h pred
      toSeq h supp
      toSeq h n
      toSeq h mv
      toSeq h ts
 fromSeq h = 
  do i <- getC h
     case i of 
       'L' -> return QTNil
       'N' -> do pred <- fromSeq h 
                 supp <- fromSeq h 
                 n    <- fromSeq h
                 mv   <- fromSeq h
                 ts   <- fromSeq h
		 let q = QTNode pred supp n mv ts
                 q `seq` return q

instance Sequential HierTree where
  toSeq h (HierNode a count b ts) = 
     do toSeq h a
	toSeq h count     
	toSeq h ts
  fromSeq h = 
     do a     <- fromSeq h
	count <- fromSeq h
	ts    <- fromSeq h
        let ht = HierNode a count False ts
        ht `seq` return ht

instance Sequential HierTreePair where
  toSeq h (HierTreePair (s,ts)) =
   do toSeqString h s
      toSeq h ts
  fromSeq h =
   do s  <- fromSeqString h
      ts <- fromSeq h
      let hier = HierTreePair (s,ts)
      hier `seq` return hier

{-
data T = L | N Int T T
         deriving (Show, Eq) -- just for testing

instance Sequential T where
   toSeq h L = putC h 'L'
   toSeq h (N n l r) = do putC h 'N'
                          toSeq h n
                          toSeq h l
                          toSeq h r

   fromSeq h
      = do what <- getC h
           case what of
              'L' -> return L
              'N' -> do n <- fromSeq h
                        l <- fromSeq h
                        r <- fromSeq h
                        return (N n l r)


mkHugeTree :: Int -> T
mkHugeTree n
   | n == 0 = L
   | n == 1 = N 1 L L
   | n > 0  = let half = n `div` 2
              in  N n (mkHugeTree half) (mkHugeTree (n - half))

-- just to force the read thing into memory
countNodes :: T -> Int
countNodes L = 0
countNodes (N _ l r) = 1 + countNodes l + countNodes r

test2 = do let tree_orig = mkHugeTree 500000
	--   putStrLn (show (countNodes tree_orig))
           h_out <- openFile "testfile2" WriteMode
           toSeq h_out tree_orig
           hClose h_out
           h_in <- openFile "testfile2" ReadMode
           tree_read <- (fromSeq h_in) :: IO T
           let n_nodes_read = countNodes tree_read
           putStrLn ("read tree has " ++ show n_nodes_read ++ " nodes")
           putStrLn ("is it same as original?: " ++ show (tree_orig == tree_read))

main = test2
-}
