$Id: PatternParser.ly,v 1.3 2003/09/08 16:30:35 afc Exp $
$Log: PatternParser.ly,v $
Revision 1.3  2003/09/08 16:30:35  afc
support for Intervals added

Revision 1.2  2002/07/16 16:29:41  ajc99
Now keeps support value too.

Revision 1.1  2002/05/27 11:27:13  ajc99
Initial revision
 

> {
> module PatternParser where
> import Char
> import ADT
> import PackedString(packString)
> }

> %name patternparser Ps
> %tokentype { Token }

> %monad { P } { thenP } { returnP }
> %lexer { lexer } { TokenEOF }

> %token 
>       name		{ TokenName $$ }
>       interval	{ TokenInterval $$ }
>       support		{ TokenSupport $$ }
>       varname		{ TokenVar $$ }
>	','		{ TokenComma }
>	'('		{ TokenOB }
>	')'		{ TokenCB }


> %%


> Ps	  : Pattern			{ [$1] }
>	  | Pattern  Ps			{ $1 : $2 }


> Pattern : support Preds 		{ ($2,$1) }

> Preds   : Pred			{ [$1] }
>         | Pred Preds			{ $1 : $2 } 

> Pred    : name '(' ArgList ')'        { PredString $1 $3 }

> ArgList : Arg                         { [$1] }
>         | Arg ',' ArgList             { $1 : $3 }

> Arg     : varname                     { Var (read $1) }
>	  | interval			{ Interval (read (fst $1)) (read (snd $1)) } 
>         | name                        { Atom (packString $1) }

> {

-----------------------------------------------------------------------------
The monad serves three purposes: 

	* it passes the input string around
	* it passes the current line number around
	* it deals with success/failure.

> data ParseResult a
>	= ParseOk a
>	| ParseFail String

> -- text, line number, filename 
> type P a = String -> Int -> String -> ParseResult a

> thenP :: P a -> (a -> P b) -> P b
> m `thenP` k = \s l n -> 
>	case m s l n of
>		ParseFail s -> ParseFail s
>		ParseOk a -> k a s l n

> returnP :: a -> P a
> returnP a = \s l n -> ParseOk a

-----------------------------------------------------------------------------

The datastructure for the tokens...

> data Token
>	= TokenName String
>	| TokenVar String
>	| TokenInterval (String,String) 
>	| TokenSupport String
>	| TokenOB
>	| TokenCB
>	| TokenComma
>	| TokenEOF

.. and a simple lexer that returns this datastructure.

> lexer :: (Token -> P a) -> P a
> lexer cont s = case s of
> 	[] -> cont TokenEOF []
>  	('\n':cs) -> \line -> lexer cont cs (line+1)
>	('-':'-':cs) -> lexer cont (dropWhile ((/=) '\n') cs)	
> 	(',':cs) -> cont TokenComma cs
> 	('(':cs) -> cont TokenOB cs
> 	(')':cs) -> cont TokenCB cs
> 	(c:cs) 
>               | isSpace c -> lexer cont cs
>               | isVarChar c -> lexVar cs
>               | isFirstNameChar c -> lexName (c:cs)
> 		| otherwise -> \line name -> error (name ++ ": character '" ++ [c] ++ "' not recognised at line " ++ show line ++ "\n")
>  where
> 	lexVar cs =
>    	    case span isDigit cs of
>		(digits,rest) -> cont (TokenVar digits) rest
> 	lexName cs =
>    	    case span isNameChar cs of
> 		(var,rest)   -> case isInterval var of
>                                 Just i -> cont (TokenInterval i) rest
>                                 Nothing -> 
>                                   if all isSupportChar var 
>                                   then cont (TokenSupport var) rest
>                                   else cont (TokenName var) rest

> isSupportChar c = isDigit c || c == '/'
> isVarChar c = c == '_'
> isFirstNameChar c = isDigit c || isAlpha c 
> isNameChar c = isFirstNameChar c || c == '-' || c == '.' || c == '/' 
>                || c == '_'


> isInterval string = let (left,right) = splitAtDash string in
>		      if not (null left) && not (null right) 
>                     && all isdouble left 
>                     && all isdouble right
>                     then Just (left,right)
>		      else Nothing

> splitAtDash [] = ([],[])
> splitAtDash ('e':'-':cs) = let (b,a) = splitAtDash cs in
>			     ('e':'-':b,a)
> splitAtDash ('-':cs) = ([],cs)
> splitAtDash (c:cs) = let (b,a) = splitAtDash cs in
>		       (c:b,a)

> isdouble c = isDigit c || c=='e' || c =='.' || c=='-'

> runPatternParser :: String -> String -> [([PredString],String)]
> runPatternParser s filename = 
>     case patternparser s 1 filename of
>		       ParseOk e -> e
>		       ParseFail s -> error (filename ++ ": " ++ s)



-----------------------------------------------------------------------------
The following functions should be defined for all parsers.

This is the overall type of the parser.

> type ParsePattern = P [([PredString],String)]
> patternparser :: ParsePattern


The next function is called when a parse error is detected.  It has
the same type as the top-level parse function.

> happyError :: P a
> happyError = \s i n -> error (
>	n ++ ": Parse error in line " ++ show (i::Int) ++ "\n")

-----------------------------------------------------------------------------


> }