$Id: SettingsParser.ly,v 3.2 2003/04/02 17:07:30 afc Exp $
$Log: SettingsParser.ly,v $
Revision 3.2  2003/04/02 17:07:30  afc
Added intervallist and NotExistsing.

Revision 3.1  2002/05/17 16:02:16  ajc99
removed Classes stuff. allows dots and hyphens in names.
supp and conf now settings.

Revision 3.0  2002/04/04 10:45:43  ajc99
Now uses hierarchical background knowledge.

Revision 2.2  2002/02/06 15:21:58  ajc99
Changed Nil to CNil for class

Revision 2.1  2001/12/07 16:21:52  ajc99
Allows number-of-times constraints, and a rulehead to be specified.

Revision 2.0  2001/12/04 18:35:24  ajc99
no change, just version change.

Revision 1.3  2001/11/06 15:48:01  ajc99
allow constants generated from bg file.

Revision 1.2  2001/11/05 17:13:05  ajc99
added comment parsing

Revision 1.1  2001/09/27 15:06:30  ajc99
Initial revision



> {
> module SettingsParser where
> import Char
> import Ratio
> import ADT
> }

> %name settingsparser Settings
> %tokentype { Token }

> %monad { P } { thenP } { returnP }
> %lexer { lexer } { TokenEOF }

> %token 
>       name		{ TokenName $$ }
>       constlist       { TokenConst }
>       constpred       { TokenConstPred }
>       consttree       { TokenConstTree }
>       intervallist	{ TokenIntervalList }
>	constraint	{ TokenConstraint }
>	key		{ TokenKey }
>	mode		{ TokenMode }
>	typewd		{ TokenTypewd }
>	support		{ TokenSupport }
>	confidence	{ TokenConfidence }
>	rulehead	{ TokenRuleHead }
>	','		{ TokenComma }
>	'('		{ TokenOB }
>	')'		{ TokenCB }
>       '.'             { TokenDot }
>       '+'		{ TokenPlus }
>       '-'		{ TokenMinus }
>	pm		{ TokenPlusMinus }
>	'^'		{ TokenNotExisting }
>	'['		{ TokenOS }
>	']'		{ TokenCS }


> %%


> Settings  : Setting				{ [$1] }
>           | Settings Setting			{ $2 : $1 } 

> Setting   : Modes				{ M $1 }
>	    | Types				{ T $1 }
>	    | Key				{ K $1 }
>	    | Constraints			{ Co $1 }
>	    | RuleHead				{ RH $1 }
>	    | Support				{ Sp $1 }
>	    | Confidence			{ Cf $1 }

----------------------------------------------

> Modes	    : Mode				{ [$1] }
>           | Modes Mode			{ $2 : $1 }

> Mode      : mode '(' name '(' PMlist ')' ')' '.' { Mode $3 $5 }

> PMlist    : ModeArg				{ [$1] }
>	    | ModeArg ',' PMlist		{ [$1] ++ $3 }

> ModeArg   : '+'				{ Plus }
>           | '-'				{ Minus }
>           | pm				{ PlusMinus } 
>	    | '^'				{ NotExisting }
>           | constlist '[' ConstList ']'	{ ConstList $3 } 
>           | constpred name			{ ConstPred $2 }
>           | consttree name			{ ConstTree $2 }
>           | intervallist '[' IntervalList ']' { IntervalList $3 }

> ConstList : name				{ [$1] }
>	    | ConstList ',' name		{ $3 : $1 }

> IntervalList : name '-' name			{ [(read $1, read $3)] }
>	       | IntervalList ',' name '-' name	{ (read $3, read $5) : $1 }

-----------------------------------------------

> Types	    : Type				{ [$1] }
>	    | Types Type			{ $2 : $1 }

> Type	    : typewd '(' name '(' TypeArgsList ')' ')' '.' { Type $3 $5 }

   TypeArgsList : name				{ if elem $1 typelist then [read $1] else error ("Settings file: " ++ $1 ++ " is not a type") }
   	       | name ',' TypeArgsList		{ if elem $1 typelist then ([read $1] ++ $3) else error ("Settings file: " ++ $1 ++ " is not a type") }

> TypeArgsList : name				{ [$1] }
>	       | name ',' TypeArgsList		{ $1 : $3 }

-----------------------------------------------

> Key	    : key '(' name '(' PMlist ')' ')' '.' { Mode $3 $5 }

-----------------------------------------------

> RuleHead  : rulehead '(' name ')' '.'		{ $3 }

-----------------------------------------------

> Constraints : Constraint			{ [$1] }
>	      | Constraints Constraint		{ $2 : $1 }

> Constraint  : constraint '(' name ',' name ')' '.' { if all isDigit $5 then ConstraintLim $3 (read $5) else ConstraintPair ($3,$5) }


> Support     : support '(' name ',' name ')' '.'     { (read $3) % (read $5) }
> Confidence  : confidence '(' name ',' name ')' '.'  { (read $3) % (read $5) }

> {

-----------------------------------------------------------------------------
The monad serves three purposes: 

	* it passes the input string around
	* it passes the current line number around
	* it deals with success/failure.

> data ParseResult a
>	= ParseOk a
>	| ParseFail String

> -- text, line number, filename 
> type P a = String -> Int -> String -> ParseResult a

> thenP :: P a -> (a -> P b) -> P b
> m `thenP` k = \s l n -> 
>	case m s l n of
>		ParseFail s -> ParseFail s
>		ParseOk a -> k a s l n

> returnP :: a -> P a
> returnP a = \s l n -> ParseOk a

-----------------------------------------------------------------------------

The datastructure for the tokens...

> data Token
>	= TokenName String
>	| TokenOB
>	| TokenCB
>	| TokenOS
>	| TokenCS
>	| TokenConst
>	| TokenConstPred
>	| TokenConstTree
>	| TokenIntervalList
>	| TokenConstraint
>	| TokenMode
>	| TokenTypewd
>	| TokenKey
>	| TokenSupport
>	| TokenConfidence
>	| TokenRuleHead
>	| TokenPlus
>	| TokenPlusMinus
>	| TokenMinus
>       | TokenNotExisting
>	| TokenComma
>	| TokenDot
>	| TokenEOF

.. and a simple lexer that returns this datastructure.

> lexer :: (Token -> P a) -> P a
> lexer cont s = case s of
> 	[] -> cont TokenEOF []
>  	('\n':cs) -> \line -> lexer cont cs (line+1)
>	('%':cs) -> lexer cont (dropWhile ((/=) '\n') cs)	
> 	('.':cs) -> cont TokenDot cs
> 	(',':cs) -> cont TokenComma cs
> 	('(':cs) -> cont TokenOB cs
> 	(')':cs) -> cont TokenCB cs
> 	('[':cs) -> cont TokenOS cs
> 	(']':cs) -> cont TokenCS cs
> 	('+':'-':cs) -> cont TokenPlusMinus cs
> 	('+':cs) -> cont TokenPlus cs
> 	('-':cs) -> cont TokenMinus cs
> 	('^':cs) -> cont TokenNotExisting cs
> 	(c:cs) 
>               | isSpace c -> lexer cont cs
>               | isNameFirstChar c -> lexVar (c:cs)
> 		| otherwise -> \line name -> error (name ++ ": character '" ++ [c] ++ "' not recognised at line " ++ show line ++ "\n")
>  where
> 	lexVar cs =
>    	    case span isNameChar cs of
>		("constlist",rest)  -> cont TokenConst rest
>		("constpred",rest)  -> cont TokenConstPred rest
>		("consttree",rest)  -> cont TokenConstTree rest
>		("intervallist",rest)  -> cont TokenIntervalList rest
>		("constraint",rest) -> cont TokenConstraint rest
>		("support",rest)    -> cont TokenSupport rest
>		("confidence",rest) -> cont TokenConfidence rest
>		("key",rest)        -> cont TokenKey rest
>		("mode",rest)       -> cont TokenMode rest
>		("type",rest)       -> cont TokenTypewd rest
>		("rulehead",rest)   -> cont TokenRuleHead rest
> 		(var,rest)          -> cont (TokenName var) rest

> isNameFirstChar c = isDigit c || isAlpha c || c == '_' 
> isNameChar c      = isDigit c || isAlpha c || c == '_' 
>                     || c == '.' || c == '-'

> runSettingsParser :: String -> String -> [Setting]
> runSettingsParser s filename = 
>             case settingsparser s 1 filename of
>		          ParseOk e -> e
>		          ParseFail s -> error (filename ++ ": " ++ s)

-----------------------------------------------------------------------------
The following functions should be defined for all parsers.

This is the overall type of the parser.

> type ParseSettings = P [Setting]
> settingsparser :: ParseSettings

The next function is called when a parse error is detected.  It has
the same type as the top-level parse function.

> happyError :: P a
> happyError = \s i n -> error (
>	n ++ ": Parse error in line " ++ show (i::Int) ++ "\n")

-----------------------------------------------------------------------------


> }