{-
$Id: Utils.hs,v 1.2 2003/09/08 16:29:15 afc Exp $
$Log: Utils.hs,v $
Revision 1.2  2003/09/08 16:29:15  afc
Line added to deal with newVarPreds

Revision 1.1  2002/07/16 16:33:18  ajc99
Initial revision


-}

module Utils( parsepatterns, Pattern, Patterns, makeAncestorLookup,
              getMaxVar, replaceAllVarsInQuery, makeAncestorFMs )

where

import ADT
import PatternParser

import FiniteMap(emptyFM,addListToFM)

---------------------------------------------------------------------

type Patterns = [Pattern]
type Pattern = Assoc

---------------------------------------------------------------------

parsepatterns :: [ModeNum] -> String -> IO( Patterns )
parsepatterns modenums filename = 
    do text <- readFile filename
       let pps = runPatternParser text filename 
       return (map (compilePattern modenums) pps)

compilePattern modenums (predstrings,supportstring) = 
   let (a,b) = supportparse supportstring
       pred  = map (compilePred modenums) predstrings
   in Assoc pred a b

compilePred modenums (PredString "notequal" args) = Pred newVarPred args
compilePred modenums (PredString name args) = 
     case lookupModeNum name modenums of 
              Just n -> Pred n args
              Nothing -> Pred (-1) args  

lookupModeNum name [] = Nothing
lookupModeNum name ((ModeNum mnum mname margs):modenums) =
  if name == mname then Just mnum else lookupModeNum name modenums

supportparse s = 
   let (a,b) = break (== '/') s
       in (read a, read (drop 1 b))


---------------------------------------------------------------------

-- we need to lookup each type of arg of prednum in the modes and use the
-- appropriate Maybe AncestorFM for each position.

-- the ancestor FMs need to have been made right from the start, and
-- linked/zipped with their names


makeAncestorLookup hier = 
   zip (hiernames hier) (map (makeAncestorFMs emptyFM) hier)
   where 
        hiernames  = map (\(HierTreePair (n,ts)) -> n)

----------------------------------------------------------------------

maxVarArgs vsofar (Var v) = if v > vsofar then v else vsofar
maxVarArgs vsofar _ = vsofar

getMaxVarArgs v args = foldl maxVarArgs v args

maxVar v (Pred n args) = getMaxVarArgs v args

getMaxVar preds = foldl maxVar 0 preds

----------------------------------------------------------------------

-- occurs check: replace all var nums with ones which won't clash

replaceVar offset (Var v) = Var (v+offset)
replaceVar offset a = a

replaceAllVarsInPred offset (Pred n args) = 
      Pred n (map (replaceVar offset) args)

replaceAllVarsInQuery offset = map (replaceAllVarsInPred offset)

----------------------------------------------------------------------

treeFlatten (HierNode name val b []) = [name]
treeFlatten (HierNode name val b ts) = name : concatMap treeFlatten ts

makeAncestorFM fm (HierNode name val b []) = fm 
makeAncestorFM fm (HierNode name val b ts) = 
   let newfm = addListToFM fm [((name,child),True) | child <- concatMap treeFlatten ts] 
   in foldl makeAncestorFM newfm ts

makeAncestorFMs fm (HierTreePair (_,hiertrees)) = foldl makeAncestorFM fm hiertrees

